import numpy as np
from sklearn.metrics.pairwise import euclidean_distances


def run(high_dim_counts, low_dim_counts):
    """
    Computes the kruskal stress for the low dimension representation
    stored at adata.obsm.key_repr

    :param adata:
    :param key_repr:
    :return:
    """
    low_dim_distances = euclidean_distances(low_dim_counts, low_dim_counts)
    high_dim_distances = euclidean_distances(high_dim_counts, high_dim_counts)
    stress = np.sqrt(
        np.square(high_dim_distances - low_dim_distances).sum()
        / np.square(low_dim_distances).sum()
    )
    return stress



import logging
import os
import warnings

import pandas as pd
import scanpy as sc
from anndata import AnnData

from . import checkatlas

EXTENSIONS_CELLRANGER = [".h5", ".mtx"]
CELLRANGER_FILE = "filtered_feature_bc_matrix.h5"
CELLRANGER_MATRIX_FILE = "matrix.mtx"
CELLRANGER_TYPE_OBSOLETE = "Cellranger < v3"
CELLRANGER_TYPE_CURRENT = "Cellranger >= v3"

logger = logging.getLogger("checkatlas")
warnings.simplefilter(action="ignore", category=FutureWarning)
warnings.simplefilter(action="ignore", category=UserWarning)
sc.settings.verbosity = 0


def detect_cellranger(atlas_path: str) -> dict:
    if atlas_path.endswith(CELLRANGER_FILE):
        atlas_info = dict()
        atlas_info[checkatlas.ATLAS_NAME_KEY] = atlas_path.split(os.sep)[-3]
        atlas_info[checkatlas.ATLAS_TYPE_KEY] = CELLRANGER_TYPE_CURRENT
        atlas_info[checkatlas.ATLAS_EXTENSION_KEY] = EXTENSIONS_CELLRANGER[0]
        atlas_info[checkatlas.ATLAS_PATH_KEY] = atlas_path
        return atlas_info
    elif atlas_path.endswith(CELLRANGER_MATRIX_FILE):
        atlas_info = dict()
        atlas_info[checkatlas.ATLAS_NAME_KEY] = atlas_path.split(os.sep)[-5]
        atlas_info[checkatlas.ATLAS_TYPE_KEY] = CELLRANGER_TYPE_OBSOLETE
        atlas_info[checkatlas.ATLAS_EXTENSION_KEY] = EXTENSIONS_CELLRANGER[1]
        atlas_info[checkatlas.ATLAS_PATH_KEY] = atlas_path
        return atlas_info
    else:
        return dict()


def read_cellranger_current(atlas_info: dict) -> AnnData:
    """
    Read cellranger files.

    Load first /outs/filtered_feature_bc_matrix.h5
    Then add (if found):
    - Clustering
    - PCA-
    - UMAP
    - TSNE
    Args:
        atlas_path (dict): info on the atlas

    Returns:
        AnnData: scanpy object from cellranger
    """
    cellranger_out_path = os.path.dirname(
        atlas_info[checkatlas.ATLAS_PATH_KEY]
    )
    cellranger_analysis_path = os.path.join(cellranger_out_path, "analysis")
    cellranger_clust_path = os.path.join(
        cellranger_analysis_path, "clustering"
    )
    cellranger_umap_path = os.path.join(cellranger_analysis_path, "umap")
    cellranger_tsne_path = os.path.join(cellranger_analysis_path, "tsne")
    cellranger_pca_path = os.path.join(cellranger_analysis_path, "pca")

    # Search graphclust
    graphclust_path = ""
    for root, dirs, files in os.walk(cellranger_clust_path):
        for dir in dirs:
            if dir.endswith("graphclust"):
                cluster_path = os.path.join(root, dir, "clusters.csv")
                if os.path.exists(cluster_path) and not root.endswith("atac"):
                    graphclust_path = cluster_path
                    break
    # Search kmeans
    kmeans_path = ""
    k_value = 0
    found_kmeans = False
    for root, dirs, files in os.walk(cellranger_clust_path):
        for dir in dirs:
            # Search the highest kmeans = 10
            dir_prefix = "kmeans_10"
            if dir_prefix in dir and not found_kmeans:
                cluster_path = os.path.join(root, dir, "clusters.csv")
                if os.path.exists(cluster_path):
                    kmeans_path = cluster_path
                    k_value = 10
                    found_kmeans = True
                    break
            # Or search the highest kmeans = 5 (for multiome atlas)
            dir_prefix = os.path.join("gex", "kmeans_5")
            if dir_prefix in os.path.join(root, dir) and not found_kmeans:
                cluster_path = os.path.join(root, dir, "clusters.csv")
                if os.path.exists(cluster_path):
                    kmeans_path = cluster_path
                    k_value = 5
                    found_kmeans = True
                    break

    # Search umap
    rna_umap = ""
    for root, dirs, files in os.walk(cellranger_umap_path):
        for file in files:
            if file.endswith("projection.csv") and not root.endswith("atac"):
                rna_umap = os.path.join(root, file)
                break

    # Search t-SNE
    rna_tsne = ""
    for root, dirs, files in os.walk(cellranger_tsne_path):
        for file in files:
            if file.endswith("projection.csv") and not root.endswith("atac"):
                rna_tsne = os.path.join(root, file)
                break

    rna_pca = ""
    for root, dirs, files in os.walk(cellranger_pca_path):
        for file in files:
            if file.endswith("projection.csv") and not root.endswith("atac"):
                rna_pca = os.path.join(root, file)
                break

    # Manage multiome cellranger files
    dim_red_path = os.path.join(
        cellranger_analysis_path, "dimensionality_reduction"
    )
    if os.path.exists(dim_red_path):
        gex_path = os.path.join(dim_red_path, "gex")
        if os.path.exists(gex_path):
            rna_umap = os.path.join(gex_path, "umap_projection.csv")
            rna_tsne = os.path.join(gex_path, "tsne_projection.csv")
            rna_pca = os.path.join(gex_path, "pca_projection.csv")

    # Read 10x h5 file
    adata = sc.read_10x_h5(atlas_info[checkatlas.ATLAS_PATH_KEY])
    adata.var_names_make_unique()

    # Add cluster
    if os.path.exists(graphclust_path):
        df_cluster = pd.read_csv(graphclust_path, index_col=0)
        adata.obs["cellranger_graphclust"] = df_cluster["Cluster"]
    if os.path.exists(kmeans_path):
        df_cluster = pd.read_csv(kmeans_path, index_col=0)
        adata.obs["cellranger_kmeans_" + str(k_value)] = df_cluster["Cluster"]

    # Add reduction
    if os.path.exists(rna_umap):
        df_umap = pd.read_csv(rna_umap, index_col=0)
        adata.obsm["X_umap"] = df_umap
    if os.path.exists(rna_tsne):
        df_tsne = pd.read_csv(rna_tsne, index_col=0)
        adata.obsm["X_tsne"] = df_tsne
    if os.path.exists(rna_pca):
        df_pca = pd.read_csv(rna_pca, index_col=0)
        adata.obsm["X_pca"] = df_pca

    return adata


def read_cellranger_obsolete(atlas_info: dict) -> AnnData:
    """
    Read cellranger files.

    Load first /outs/filtered_feature_bc_matrix.h5
    Then add (if found):
    - Clustering
    - PCA-
    - UMAP
    - TSNE
    Args:
        atlas_path (dict): info on the atlas

    Returns:
        AnnData: scanpy object from cellranger
    """
    cellranger_path = atlas_info[checkatlas.ATLAS_PATH_KEY].replace(
        CELLRANGER_MATRIX_FILE, ""
    )
    cellranger_out_path = os.path.join(cellranger_path, os.pardir, os.pardir)
    cellranger_analysis_path = os.path.join(
        cellranger_out_path, "analysis_csv"
    )

    cellranger_umap_path = os.path.join(cellranger_analysis_path, "umap")
    cellranger_tsne_path = os.path.join(cellranger_analysis_path, "tsne")
    cellranger_pca_path = os.path.join(cellranger_analysis_path, "pca")
    print(cellranger_out_path)
    print(cellranger_analysis_path)
    print(cellranger_umap_path)

    # Search graphclust
    graphclust_path = ""
    for root, dirs, files in os.walk(cellranger_out_path):
        for dir in dirs:
            if dir.endswith("graphclust"):
                cluster_path = os.path.join(root, dir, "clusters.csv")
                if os.path.exists(cluster_path):
                    graphclust_path = cluster_path
                    break
    # Search kmeans
    kmeans_path = ""
    k_value = 0
    for root, dirs, files in os.walk(cellranger_out_path):
        for dir in dirs:
            if dir.endswith("kmeans"):
                # Search the highest kmeans from 15 to 3
                for k in reversed(range(3, 16)):
                    cluster_path = os.path.join(
                        root, dir, str(k) + "_clusters", "clusters.csv"
                    )
                    if os.path.exists(cluster_path):
                        kmeans_path = cluster_path
                        k_value = k
                        break

    rna_umap = os.path.join(cellranger_umap_path, "projection.csv")
    rna_tsne = os.path.join(cellranger_tsne_path, "projection.csv")
    rna_pca = os.path.join(cellranger_pca_path, "projection.csv")

    # get matrix folder
    matrix_folder = os.path.dirname(atlas_info[checkatlas.ATLAS_PATH_KEY])
    adata = sc.read_10x_mtx(matrix_folder)
    adata.var_names_make_unique()

    # Add cluster
    if os.path.exists(graphclust_path):
        df_cluster = pd.read_csv(graphclust_path, index_col=0)
        adata.obs["cellranger_graphclust"] = df_cluster["Cluster"]
    if os.path.exists(kmeans_path):
        df_cluster = pd.read_csv(kmeans_path, index_col=0)
        adata.obs["cellranger_kmeans_" + str(k_value)] = df_cluster["Cluster"]

    # Add reduction
    if os.path.exists(rna_umap):
        df_umap = pd.read_csv(rna_umap, index_col=0)
        adata.obsm["X_umap"] = df_umap
    if os.path.exists(rna_tsne):
        df_tsne = pd.read_csv(rna_tsne, index_col=0)
        if len(df_tsne) == len(adata):
            adata.obsm["X_tsne"] = df_tsne
    if os.path.exists(rna_pca):
        df_pca = pd.read_csv(rna_pca, index_col=0)
        if len(df_pca) == len(adata):
            adata.obsm["X_pca"] = df_pca
    return adata



import logging

import rpy2.robjects as ro
import rpy2.robjects as robjects
from sklearn.preprocessing import LabelEncoder

from . import annot, cluster, dimred

METRICS_CLUST = cluster.__all__
METRICS_ANNOT = annot.__all__
METRICS_DIMRED = dimred.__all__

logger = logging.getLogger("checkatlas")

R_ANNOT = robjects.r(
    "type <- function(seurat, obs_key){ "
    "return(seurat[[obs_key]][[obs_key]])}"
)
R_REDUCTION = robjects.r(
    "reduc <- function(seurat, obsm_key){"
    " return(Embeddings(object = seurat, "
    "reduction = obsm_key))}"
)


def calc_metric_cluster_scanpy(
    metric, adata, obs_key, obsm_key_representation
):
    if metric in METRICS_CLUST:
        metric_module = getattr(cluster, metric)
        annotations = adata.obs[obs_key]
        if obsm_key_representation != "":
            count_repr = adata.obsm[obsm_key_representation]
            return metric_module.run(count_repr, annotations)
        else:
            original_count = adata.X.toarray()
            return metric_module.run(original_count, annotations)
    else:
        logger.warning(
            f"{metric} is not a recognized "
            f"cluster metric.\n"
            f"List of clustering metrics: {METRICS_CLUST}"
        )
        return -1


def calc_metric_cluster_seurat(
    metric, seurat, obs_key, obsm_key_representation
):
    if metric in METRICS_CLUST:
        metric_module = getattr(cluster, metric)
        annotations = ro.conversion.rpy2py(R_ANNOT(seurat, obs_key))
        count_repr = ro.conversion.rpy2py(
            R_REDUCTION(seurat, obsm_key_representation)
        )
        return metric_module.run(count_repr, annotations)
    else:
        logger.warning(
            f"{metric} is not a recognized "
            f"cluster metric.\n"
            f"List of clustering metrics: {METRICS_CLUST}"
        )
        return -1


def calc_metric_annot_scanpy(metric, adata, obs_key, ref_obs):
    if metric in METRICS_ANNOT:
        metric_module = getattr(annot, metric)
        annotation = adata.obs[obs_key]
        ref_annotation = adata.obs[ref_obs]
        annotation, ref_annotation = annotation_to_num(
            annotation, ref_annotation
        )
        return metric_module.run(annotation, ref_annotation)
    else:
        logger.warning(
            f"{metric} is not a recognized annotation metric."
            f"\nList of annotation metrics: {METRICS_ANNOT}"
        )
        return -1


def calc_metric_annot_seurat(metric, seurat, obs_key, ref_obs):
    if metric in METRICS_ANNOT:
        metric_module = getattr(annot, metric)
        annotation = ro.conversion.rpy2py(R_ANNOT(seurat, obs_key))
        ref_annotation = ro.conversion.rpy2py(R_ANNOT(seurat, ref_obs))
        # annotation, ref_annotation = annotation_to_num(
        #    annotation, ref_annotation
        # )
        return metric_module.run(annotation, ref_annotation)
    else:
        logger.warning(
            f"{metric} is not a recognized annotation metric."
            f"\nList of annotation metrics: {METRICS_ANNOT}"
        )
        return -1


def calc_metric_dimred(metric, adata, obsm_key):
    if metric in METRICS_DIMRED:
        metric_module = getattr(dimred, metric)
        high_dim_counts = adata.X
        low_dim_counts = adata.obsm[obsm_key]
        return metric_module.run(high_dim_counts, low_dim_counts)
    else:
        logger.warning(
            f"{metric} is not a recognized "
            f"dimensionality reduction metric."
            f"\nList of dim. red. metrics: {METRICS_DIMRED}"
        )
        return -1


def annotation_to_num(annotation, ref_annotation):
    """
    Transforms the annotations from categorical to numerical

    Parameters
    ----------
    adata
    partition_key
    reference

    Returns
    -------

    """
    annotation = annotation.to_numpy()
    ref_annotation = ref_annotation.to_numpy()
    le = LabelEncoder()
    le.fit(annotation)
    annotation = le.transform(annotation)
    le2 = LabelEncoder()
    le2.fit(ref_annotation)
    ref_annotation = le2.transform(ref_annotation)
    return annotation, ref_annotation



from sklearn.metrics import silhouette_score


def run(count_repr, annotations):
    """

    :param count_repr:
    :param annotations:
    :return:
    """
    return silhouette_score(count_repr, annotations)



import argparse

import yaml

from .. import atlas
from ..metrics import annot, cluster, dimred


def create_parser():
    parser = argparse.ArgumentParser(
        prog="checkatlas",
        usage="checkatlas [OPTIONS] your_search_folder/",
        description="CheckAtlas is a one liner tool to check the "
        "quality of your single-cell atlases. For "
        "every atlas, it produces the quality control "
        "tables and figures which can be then processed "
        "by multiqc. CheckAtlas is able to load Scanpy, "
        "Seurat, and CellRanger files.",
        epilog="Enjoy the checkatlas functionality!",
    )

    # All Program arguments
    # main_options = parser.add_argument_group("Main arguments")
    parser.add_argument(
        "path",
        type=str,
        help="Required argument: Your folder containing "
        "Scanpy, CellRanger and Seurat atlasesv",
        default=".",
    )

    parser.add_argument(
        "--plot_celllimit",
        type=int,
        default=10000,
        help="Set the maximum number of cells"
        "to plot in QC, UMAP, t-SNE, etc...."
        "If plot_celllimit=0, no limit will"
        "be applied.",
    )

    parser.add_argument(
        "--nextflow",
        type=int,
        default=0,
        help="Activate Nextflow and specify the number of threads to use. \n"
        "Example: --nextflow 8 (for 8 threads)",
    )

    parser.add_argument(
        "-d",
        "--debug",
        action="store_true",
        help="Print out all debug messages.",
    )
    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=f"Checkatlas {get_version()}",
        help="Display checkatlas version.",
    )

    # Pipeline arguments
    pipeline_options = parser.add_argument_group("Manage checkatlas pipeline")
    pipeline_options.add_argument(
        "-na",
        "--NOADATA",
        action="store_true",
        help="Do not Create adata summary tables.",
    )
    pipeline_options.add_argument(
        "-nq",
        "--NOQC",
        action="store_true",
        help="Do not produce any quality control figures or tables.",
    )
    pipeline_options.add_argument(
        "-nr",
        "--NOREDUCTION",
        action="store_true",
        help="Do not Produce UMAP and t-SNE figures.",
    )
    pipeline_options.add_argument(
        "-nm",
        "--NOMETRIC",
        action="store_true",
        help="Do not calculate any metric.",
    )
    pipeline_options.add_argument(
        "-nc", "--NOMULTIQC", action="store_true", help="Do not run multiqc."
    )

    # Arguments linked to QC
    qc_options = parser.add_argument_group("QC options")
    qc_options.add_argument(
        "--qc_display",
        nargs="+",
        type=str,
        default=[
            "violin_plot",
            "total_counts",
            "n_genes_by_counts",
            "pct_counts_mt",
        ],
        help="List of QC to display. "
        "Available qc = violin_plot, total_counts, "
        "n_genes_by_counts, pct_counts_mt. "
        "Default: --qc_display violin_plot total_counts "
        "n_genes_by_counts pct_counts_mt",
    )

    # Arguments linked to metric
    metric_options = parser.add_argument_group("Metric options")
    metric_options.add_argument(
        "--obs_cluster",
        nargs="+",
        type=str,
        default=atlas.OBS_CLUSTERS,
        help="List of obs from the adata file to "
        "use in the clustering metric calculus."
        "Example: --obs_cluster celltype leuven seurat_clusters",
    )
    metric_options.add_argument(
        "--metric_cluster",
        nargs="+",
        type=str,
        # default=["silhouette", "davies_bouldin"],
        default=["davies_bouldin"],
        help="Specify the list of clustering metrics to calculate.\n"
        "   Example: --metric_cluster silhouette davies_bouldin\n"
        f"   List of cluster metrics: {cluster.__all__}",
    )
    metric_options.add_argument(
        "--metric_annot",
        nargs="+",
        type=str,
        default=[],
        # default=["rand_index"],
        help=f"Specify the list of clustering metrics to calculate."
        f"   Example: --metric_annot rand_index"
        f"   List of annotation metrics: {annot.__all__}",
    )
    metric_options.add_argument(
        "--metric_dimred",
        nargs="+",
        type=str,
        # default=["kruskal_stress"],
        default=[],
        help="Specify the list of dimensionality reduction "
        "metrics to calculate.\n"
        "   Example: --metric_dimred kruskal_stress\n"
        f"   List of dim. red. metrics: {dimred.__all__}",
    )
    metric_options.add_argument(
        "--TEST_ALLMETRICS",
        action="store_true",
        help="Run the pipeline with all metrics available.\n"
        f"   List of cluster metrics: {cluster.__all__}\n"
        f"   List of annotation metrics: {annot.__all__}\n"
        f"   List of dim. red. metrics: {dimred.__all__}",
    )
    return parser


def load_arguments(args, yaml_name):
    """
    Load all args from a yaml file.

    :param args:
    :param yaml_name:
    :return: args
    """
    with open(yaml_name, "r") as config_file:
        yaml_args = yaml.load(config_file, Loader=yaml.FullLoader)
        arg_dict = args.__dict__
        for key, value in yaml_args.items():
            if isinstance(value, list):
                arg_dict[key] = list()
                for v in value:
                    arg_dict[key].append(v)
            else:
                arg_dict[key] = value
        return args


def save_arguments(args, yaml_name):
    """
    Save all args to a yaml file. Only use this
    function to create example yaml config files.

    :param args:
    :param yaml_name:
    :return:
    """
    with open(yaml_name, "w") as config_file:
        yaml.dump(args.__dict__, config_file)


def get_version():
    """
    Get version of checkatlas from checkatlas/VERSION file
    :return: checkatlas version
    """
    return "Need to Fix version reading!"
    # script_path = os.path.dirname(os.path.realpath(__file__))
    # version_file = os.path.join(script_path, "VERSION")
    # with open(version_file, "r") as version:
    #     return version.readlines()[0].strip()



import logging
import os

import pandas as pd

from . import atlas, cellranger, seurat
from .utils import files as chk_files
from .utils import folders

"""
checkatlas base module.
This is the principal module of the checkatlas project.

"""

PROCESS_TYPE = [
    "summary",
    "qc",
    "metric_cluster",
    "metric_annot",
    "metric_dimred",
]

TSV_EXTENSION = ".tsv"
QC_FIG_EXTENSION = "_qc.png"
UMAP_EXTENSION = "_umap.png"
TSNE_EXTENSION = "_tsne.png"

ATLAS_NAME_KEY = "Atlas_name"
ATLAS_TYPE_KEY = "Atlas_type"
ATLAS_EXTENSION_KEY = "Atlas_extension"
ATLAS_PATH_KEY = "Atlas_path"
ATLAS_TABLE_HEADER = [
    ATLAS_NAME_KEY,
    ATLAS_TYPE_KEY,
    ATLAS_EXTENSION_KEY,
    ATLAS_PATH_KEY,
]

openpng_html_script = """
<script>
function openPNG(evt, pngName, tablinks_id, tabcontent_id) {
  var i, tabcontent, tablinks;
  tabcontent = document.getElementsByClassName(tabcontent_id);
  for (i = 0; i < tabcontent.length; i++) {
    tabcontent[i].style.display = "none";
  }
  tablinks = document.getElementsByClassName(tablinks_id);
  for (i = 0; i < tablinks.length; i++) {
    tablinks[i].className = tablinks[i].className.replace(" active", "");
  }
  document.getElementById(pngName).style.display = "block";
  evt.currentTarget.className += " active";
}
</script>
"""

logger = logging.getLogger("checkatlas")


def list_scanpy_atlases(checkatlas_path: str) -> None:
    # Get all files with matching extension
    atlas_list = list()
    for root, dirs, files in os.walk(checkatlas_path):
        for file in files:
            if file.endswith(atlas.ANNDATA_EXTENSION):
                atlas_list.append(os.path.join(root, file))

    # Filter the lists keepng only atlases
    clean_scanpy_list = list()
    for atlas_path in atlas_list:
        atlas_info = atlas.detect_scanpy(atlas_path)
        if len(atlas_info) != 0:
            # detect if its a cellranger output
            logger.debug(
                f"Include Atlas: {atlas_info[ATLAS_NAME_KEY]}"
                f" of type {atlas_info[ATLAS_TYPE_KEY]}"
                f"from {atlas_info[ATLAS_PATH_KEY]}"
            )
            clean_scanpy_list.append(atlas_info)
    logger.info(
        f"Found {len(clean_scanpy_list)} potential "
        f"seurat files with .rds extension"
    )
    # Save the list of atlas taken into account
    chk_files.save_list_scanpy(clean_scanpy_list, checkatlas_path)


def list_cellranger_atlases(checkatlas_path: str) -> None:
    # Get all files with matching extension
    EXTENSIONS = [
        cellranger.CELLRANGER_FILE,
        cellranger.CELLRANGER_MATRIX_FILE,
    ]
    atlas_list = list()
    for root, dirs, files in os.walk(checkatlas_path):
        for file in files:
            for extension in EXTENSIONS:
                if file.endswith(extension):
                    atlas_list.append(os.path.join(root, file))

    # Filter the lists keepng only atlases
    clean_cellranger_list = list()
    for atlas_path in atlas_list:
        atlas_info = cellranger.detect_cellranger(atlas_path)
        if len(atlas_info) != 0:
            # detect if its a cellranger output
            logger.debug(
                f"Include Atlas: {atlas_info[ATLAS_NAME_KEY]}"
                f" of type {atlas_info[ATLAS_TYPE_KEY]}"
                f"from {atlas_info[ATLAS_PATH_KEY]}"
            )
            clean_cellranger_list.append(atlas_info)
    logger.info(
        f"Found {len(clean_cellranger_list)} potential "
        f"seurat files with .rds extension"
    )
    # Save the list of atlas taken into account
    chk_files.save_list_cellranger(clean_cellranger_list, checkatlas_path)


def list_seurat_atlases(checkatlas_path: str) -> None:
    # Get all files with matching extension
    atlas_list = list()
    for root, dirs, files in os.walk(checkatlas_path):
        for file in files:
            if file.endswith(seurat.SEURAT_EXTENSION):
                atlas_list.append(os.path.join(root, file))

    # Filter the lists keepng only atlases
    clean_seurat_list = list()
    for atlas_path in atlas_list:
        atlas_info = seurat.detect_seurat(atlas_path)
        if len(atlas_info) != 0:
            # detect if its a seurat output
            logger.debug(
                f"Include Atlas: {atlas_info[ATLAS_NAME_KEY]}"
                f" of type {atlas_info[ATLAS_TYPE_KEY]}"
                f"from {atlas_info[ATLAS_PATH_KEY]}"
            )
            clean_seurat_list.append(atlas_info)
    logger.info(
        f"Found {len(clean_seurat_list)} potential "
        f"seurat files with .rds extension"
    )
    # Save the list of atlas taken into account
    chk_files.save_list_seurat(clean_seurat_list, checkatlas_path)


def read_list_atlases(checkatlas_path: str) -> tuple:
    clean_scanpy_list = pd.DataFrame()
    clean_cellranger_list = pd.DataFrame()
    clean_seurat_list = pd.DataFrame()
    if os.path.exists(chk_files.get_table_scanpy_path(checkatlas_path)):
        clean_scanpy_list = pd.read_csv(
            chk_files.get_table_scanpy_path(checkatlas_path)
        )
        clean_scanpy_list.index = clean_scanpy_list[ATLAS_NAME_KEY]
    if os.path.exists(chk_files.get_table_cellranger_path(checkatlas_path)):
        clean_cellranger_list = pd.read_csv(
            chk_files.get_table_cellranger_path(checkatlas_path)
        )
        clean_cellranger_list.index = clean_cellranger_list[ATLAS_NAME_KEY]
    if os.path.exists(chk_files.get_table_seurat_path(checkatlas_path)):
        clean_seurat_list = pd.read_csv(
            chk_files.get_table_seurat_path(checkatlas_path)
        )
        clean_seurat_list.index = clean_seurat_list[ATLAS_NAME_KEY]
    return clean_scanpy_list, clean_cellranger_list, clean_seurat_list


def generate_fig_html(checkatlas_path: str, type_viz: str):
    logger.info("Generate html reports with all figs for {type}")
    if type_viz == "qc":
        # data is a dictionnary wth key = data_name and value = fig_path
        qc_dict = dict()
        qc_fig_path = folders.get_folder(checkatlas_path, folders.QC_FIG)
        for qc_fig in os.listdir(qc_fig_path):
            if qc_fig.endswith(".png"):
                atlas_name = os.path.splitext(os.path.basename(qc_fig))[0]
                qc_dict[atlas_name] = os.path.join("violin", qc_fig)
        if len(qc_dict) > 0:
            html = create_img_html_content(type_viz, qc_dict)
            with open(
                chk_files.get_html_qc_report_path(checkatlas_path), "w"
            ) as html_report:
                html_report.write(html)
    elif type_viz == "reductions":
        umap_dict = dict()
        umap_fig_path = folders.get_folder(checkatlas_path, folders.UMAP)
        for umap_fig in os.listdir(umap_fig_path):
            if umap_fig.endswith(".png"):
                atlas_name = os.path.splitext(os.path.basename(umap_fig))[0]
                umap_dict[atlas_name] = os.path.join("umap", umap_fig)
        if len(umap_dict) > 0:
            html = create_img_html_content(type_viz, umap_dict)
            with open(
                chk_files.get_html_umap_report_path(checkatlas_path), "w"
            ) as html_report:
                html_report.write(html)

        tsne_dict = dict()
        tsne_fig_path = folders.get_folder(checkatlas_path, folders.TSNE)
        for tsne_fig in os.listdir(tsne_fig_path):
            if tsne_fig.endswith(".png"):
                atlas_name = os.path.splitext(os.path.basename(tsne_fig))[0]
                tsne_dict[atlas_name] = os.path.join("tsne", tsne_fig)
        if len(tsne_dict) > 0:
            html = create_img_html_content(type_viz, tsne_dict)
            with open(
                chk_files.get_html_tsne_report_path(checkatlas_path), "w"
            ) as html_report:
                html_report.write(html)
    else:
        logger.error(f"Type of vsualization not recognized {type_viz}")


def create_img_html_content(type_viz, data):
    html_content = f"""
            <!DOCTYPE html>
            <html lang="en">
            <head>Custom content
            with all {type_viz} plots found
            in your atlases</head>
            <body><br><br>
            <div class="tab">\n"""
    counter = 0
    tablinks = type_viz + "_tablinks"
    tabcontent = type_viz + "_tabcontent"
    for atlas_name, fig_path in data.items():
        class_tab = tablinks
        if counter == 0:
            class_tab = tablinks + " active"
        html_content += add_selection_img_button(
            class_tab, type_viz, atlas_name, tablinks, tabcontent
        )
        counter += 1
    html_content += """</div>"""

    counter = 0
    for atlas_name, fig_path in data.items():
        style = "display: none;"
        if counter == 0:
            style = "display: block;"
        html_content += add_div_img(
            fig_path, type_viz, atlas_name, style, tabcontent
        )
        counter += 1

    html_content += openpng_html_script
    html_content += """
        </body>
        </html>
        """

    return html_content


def add_selection_img_button(
    class_tab, type_viz, atlas_name, tablinks, tabcontent
):
    return (
        """  <button class=\""""
        + class_tab
        + """\" onclick="openPNG(event, '"""
        + type_viz
        + "_"
        + atlas_name
        + """',
                            '"""
        + tablinks
        + "','"
        + tabcontent
        + """')">"""
        + atlas_name
        + """</button>\n"""
    )


def add_div_img(path_fig, type_viz, atlas_name, style, tabcontent):
    return (
        """<div id=\""""
        + type_viz
        + "_"
        + atlas_name
        + """\" class=\""""
        + tabcontent
        + """\" style =\""""
        + style
        + """\"><img src=\""""
        + path_fig
        + """\" >\n</div>\n"""
    )


if __name__ == "__main__":
    path = "/data/analysis/data_becavin/checkatlas_test/tuto"
    atlas_info = {
        "Atlas_name": "pbmc_6k_v1_v1",
        "Atlas_type": "Cellranger < v3",
        "Atlas_extension": ".mtx",
        "Atlas_path": "/data/analysis/data_becavin/"
        "checkatlas_test/tuto/data5/"
        "pbmc_6k_v1_v1/outs/"
        "filtered_matrices_mex/hg19/matrix.mtx",
    }

    """ atlas_info = {'Atlas_name': 'pbmc_5k_v3_v3',
                   'Atlas_type': 'Cellranger >= v3',
                     'Atlas_extension': '.h5',
                       'Atlas_path':
                       '/data/analysis/data_becavin/'
                       'checkatlas_test/tuto/data5/'
                       'pbmc_5k_v3_v3/outs/'
                       '5k_pbmc_v3_filtered_feature_bc_matrix.h5'}

    atlas_info = {'Atlas_name': 'pbmc_5k_v3_v7',
                   'Atlas_type': 'Cellranger >= v3',
                     'Atlas_extension': '.h5',
                       'Atlas_path':
                       '/data/analysis/data_becavin/'
                       'checkatlas_test/tuto/data5/'
                       'pbmc_5k_v3_v7/outs/'
                       'SC3pv3_GEX_Human_PBMC_filtered_feature_bc_matrix.h5'}

    atlas_info = {'Atlas_name': 'pbmc_3k_multiome',
                   'Atlas_type': 'Cellranger >= v3',
                     'Atlas_extension': '.h5',
                       'Atlas_path':
                       '/data/analysis/data_becavin/'
                       'checkatlas_test/tuto/data5/'
                       'pbmc_3k_multiome/outs/'
                       'pbmc_unsorted_3k_filtered_feature_bc_matrix.h5'} """

    adata = atlas.read_atlas(atlas_info)
    # atlas_list = list_atlases(path)



import logging
import os

from checkatlas.utils import files

from . import checkatlas
from .utils import checkatlas_workflow_arguments, folders

PROCESS_TYPE = ["list_scanpy", "list_cellranger", "list_seurat", "html_report"]


def main() -> None:  # pragma: no cover
    """
    The main function executes on commands:
    `python -m checkatlas` and `$ checkatlas `.

    This is checkatlas entry point.

    Arguments are managed here
    Search fo atlases is managed here
    Then checkatlas is ran with the list of atlases found

    Returns:
        None

    """
    # Set up logging
    logger = logging.getLogger("checkatlas")
    logging.basicConfig(format="|--- %(levelname)-8s %(message)s")

    parser = checkatlas_workflow_arguments.create_parser()

    # Parse all args
    args = parser.parse_args()

    # Set logger level
    if args.debug:
        logger.setLevel(getattr(logging, "DEBUG"))
    else:
        logger.setLevel(getattr(logging, "INFO"))

    logger.debug(f"Program arguments: {args}")

    process = args.process
    if process == "list_scanpy":
        logger.debug(f"Search path {args.path} with checkatlas_workflow")
        logger.debug(f"Transform path to absolute:{args.path}")
        args.path = os.path.abspath(args.path)
        logger.debug(f"Check checkatlas folders in:{args.path}")
        folders.checkatlas_folders(args.path)
        logger.info("Searching Seurat, Cellranger and Scanpy files")
        checkatlas.list_scanpy_atlases(args.path)
    elif process == "list_cellranger":
        logger.debug(f"Search path {args.path} with checkatlas_workflow")
        logger.debug(f"Transform path to absolute:{args.path}")
        args.path = os.path.abspath(args.path)
        logger.debug(f"Check checkatlas folders in:{args.path}")
        folders.checkatlas_folders(args.path)
        logger.info("Searching Seurat, Cellranger and Scanpy files")
        checkatlas.list_cellranger_atlases(args.path)
    elif process == "list_seurat":
        logger.debug(f"Search path {args.path} with checkatlas_workflow")
        logger.debug(f"Transform path to absolute:{args.path}")
        args.path = os.path.abspath(args.path)
        logger.debug(f"Check checkatlas folders in:{args.path}")
        folders.checkatlas_folders(args.path)
        logger.info("Searching Seurat, Cellranger and Scanpy files")
        checkatlas.list_seurat_atlases(args.path)
    elif process == "html_report":
        logger.info(
            "Generate QC plots html report in "
            f"{files.get_html_qc_report_path(args.path)}"
        )
        checkatlas.generate_fig_html(args.path, "qc")
        logger.info(
            "Generate UMAP plots html report in "
            f"{files.get_html_umap_report_path(args.path)}"
        )
        checkatlas.generate_fig_html(args.path, "reductions")
    else:
        logger.debug("TO DO : Spatial Transcriptomics not yet managed.")



import os

import pandas as pd

from checkatlas import checkatlas

from . import folders


def get_file_path(
    atlas_name: str, folder: str, extension: str, path: str
) -> str:
    """_summary_

    Args:
        atlas_name (str): _description_
        args (argparse.Namespace): _description_

    Returns:
        str: _description_
    """
    csv_path = os.path.join(
        folders.get_folder(path, folder),
        atlas_name + extension,
    )
    return csv_path


def get_table_scanpy_path(checkatlas_path: str) -> str:
    file_path = os.path.join(
        folders.get_workingdir(checkatlas_path), "List_scanpy.csv"
    )
    return file_path


def get_table_cellranger_path(checkatlas_path: str) -> str:
    file_path = os.path.join(
        folders.get_workingdir(checkatlas_path), "List_cellranger.csv"
    )
    return file_path


def get_table_seurat_path(checkatlas_path: str) -> str:
    file_path = os.path.join(
        folders.get_workingdir(checkatlas_path), "List_seurat.csv"
    )
    return file_path


def save_list_scanpy(clean_scanpy_list: list, checkatlas_path: str) -> None:
    df_summary = pd.DataFrame(columns=checkatlas.ATLAS_TABLE_HEADER)
    for table_info in clean_scanpy_list:
        df_summary.loc[
            table_info[checkatlas.ATLAS_NAME_KEY]
        ] = table_info.values()
    df_summary.to_csv(
        get_table_scanpy_path(checkatlas_path), index=False, sep=","
    )


def save_list_cellranger(
    clean_cellranger_list: list, checkatlas_path: str
) -> None:
    df_summary = pd.DataFrame(columns=checkatlas.ATLAS_TABLE_HEADER)
    for table_info in clean_cellranger_list:
        df_summary.loc[
            table_info[checkatlas.ATLAS_NAME_KEY]
        ] = table_info.values()
    df_summary.to_csv(
        get_table_cellranger_path(checkatlas_path), index=False, sep=","
    )


def save_list_seurat(clean_seurat_list: list, checkatlas_path: str) -> None:
    df_summary = pd.DataFrame(columns=checkatlas.ATLAS_TABLE_HEADER)
    for table_info in clean_seurat_list:
        df_summary.loc[
            table_info[checkatlas.ATLAS_NAME_KEY]
        ] = table_info.values()
    df_summary.to_csv(
        get_table_seurat_path(checkatlas_path), index=False, sep=","
    )


def get_html_qc_report_path(checkatlas_path: str):
    return os.path.join(
        folders.get_workingdir(checkatlas_path),
        "Checkatlas_QC_report_mqc.html",
    )


def get_html_umap_report_path(checkatlas_path: str):
    return os.path.join(
        folders.get_workingdir(checkatlas_path),
        "Checkatlas_UMAP_report_mqc.html",
    )


def get_html_tsne_report_path(checkatlas_path: str):
    return os.path.join(
        folders.get_workingdir(checkatlas_path),
        "Checkatlas_tSNE_report_mqc.html",
    )



import logging

from . import atlas, cellranger, checkatlas, seurat
from .utils import checkatlas_arguments


def main() -> None:  # pragma: no cover
    """
    The main function executes on commands:
    `python -m checkatlas` and `$ checkatlas `.

    This is checkatlas entry point.

    Arguments are managed here
    Search fo atlases is managed here
    Then checkatlas is ran with the list of atlases found

    Returns:
        None

    """
    # Set up logging
    logger = logging.getLogger("checkatlas")
    logging.basicConfig(format="|--- %(levelname)-8s %(message)s")

    parser = checkatlas_arguments.create_parser()

    # Parse all args
    args = parser.parse_args()

    # Set logger level
    if args.debug:
        logger.setLevel(getattr(logging, "DEBUG"))
    else:
        logger.setLevel(getattr(logging, "INFO"))

    logger.debug(f"Program arguments: {args}")

    #   ######    Run Checkatlas   #########
    (
        clean_scanpy_list,
        clean_cellranger_list,
        clean_seurat_list,
    ) = checkatlas.read_list_atlases(args.path)
    clean_scanpy_list = clean_scanpy_list.to_dict("index")
    clean_cellranger_list = clean_cellranger_list.to_dict("index")
    clean_seurat_list = clean_seurat_list.to_dict("index")

    # get atlas_info
    if args.atlas_name in clean_scanpy_list:
        atlas_info = clean_scanpy_list[args.atlas_name]
    elif args.atlas_name in clean_cellranger_list:
        atlas_info = clean_cellranger_list[args.atlas_name]
    elif args.atlas_name in clean_seurat_list:
        atlas_info = clean_seurat_list[args.atlas_name]
    else:
        logger.error(f"Cannot found {args.atlas_name}")
    logger.debug(f"Found atlas: {atlas_info}")
    # Run process
    process = args.process
    atlas_type = atlas_info[checkatlas.ATLAS_TYPE_KEY]
    print(atlas_info)
    if (
        atlas_type == atlas.ANNDATA_TYPE
        or atlas_type == cellranger.CELLRANGER_TYPE_CURRENT
        or atlas_type == cellranger.CELLRANGER_TYPE_OBSOLETE
    ):
        if process == checkatlas.PROCESS_TYPE[0]:
            adata = atlas.read_atlas(atlas_info)
            adata = atlas.clean_scanpy_atlas(adata, atlas_info)
            atlas.create_summary_table(adata, atlas_info, args)
            atlas.create_anndata_table(adata, atlas_info, args)
            atlas.create_umap_fig(adata, atlas_info, args)
            atlas.create_tsne_fig(adata, atlas_info, args)
        elif process == checkatlas.PROCESS_TYPE[1]:
            adata = atlas.read_atlas(atlas_info)
            adata = atlas.clean_scanpy_atlas(adata, atlas_info)
            atlas.create_qc_tables(adata, atlas_info, args)
            atlas.create_qc_plots(adata, atlas_info, args)
        elif process == checkatlas.PROCESS_TYPE[2]:
            adata = atlas.read_atlas(atlas_info)
            adata = atlas.clean_scanpy_atlas(adata, atlas_info)
            atlas.create_metric_cluster(adata, atlas_info, args)
        elif process == checkatlas.PROCESS_TYPE[3]:
            adata = atlas.read_atlas(atlas_info)
            adata = atlas.clean_scanpy_atlas(adata, atlas_info)
            atlas.create_metric_annot(adata, atlas_info, args)
        elif process == checkatlas.PROCESS_TYPE[4]:
            adata = atlas.read_atlas(atlas_info)
            adata = atlas.clean_scanpy_atlas(adata, atlas_info)
            atlas.create_metric_dimred(adata, atlas_info, args)
    elif atlas_type == seurat.SEURAT_TYPE:
        if process == checkatlas.PROCESS_TYPE[0]:
            seurat_data = seurat.read_atlas(atlas_info)
            seurat.create_summary_table(seurat_data, atlas_info, args)
            seurat.create_anndata_table(seurat_data, atlas_info, args)
            seurat.create_umap_fig(seurat_data, atlas_info, args)
            seurat.create_tsne_fig(seurat_data, atlas_info, args)
        elif process == checkatlas.PROCESS_TYPE[1]:
            seurat_data = seurat.read_atlas(atlas_info)
            seurat.create_qc_tables(seurat_data, atlas_info, args)
            seurat.create_qc_plots(seurat_data, atlas_info, args)
        elif process == checkatlas.PROCESS_TYPE[2]:
            seurat_data = seurat.read_atlas(atlas_info)
            seurat.create_metric_cluster(seurat_data, atlas_info, args)
        elif process == checkatlas.PROCESS_TYPE[3]:
            seurat_data = seurat.read_atlas(atlas_info)
            seurat.create_metric_annot(seurat_data, atlas_info, args)
        elif process == checkatlas.PROCESS_TYPE[4]:
            seurat_data = seurat.read_atlas(atlas_info)
            seurat.create_metric_dimred(seurat_data, atlas_info, args)
    else:
        logger.debug("TO DO : Spatial Transcriptomics not yet managed.")



import logging
import os

WORKING_DIR = "checkatlas_files"
SUMMARY = "summary"
ANNDATA = "adata"
QC = "qc"
QC_FIG = "violin"
UMAP = "umap"
TSNE = "tsne"
CLUSTER = "cluster"
ANNOTATION = "annotation"
DIMRED = "dimred"
SPECI = "specificity"
TEMP = "temp"
NEXTFLOW = "temp/nextflow"

MULTIQC_FOLDER = "CheckAtlas_MultiQC"

DICT_FOLDER = {
    SUMMARY: SUMMARY,
    ANNDATA: ANNDATA,
    QC: QC,
    QC_FIG: QC_FIG,
    UMAP: UMAP,
    TSNE: TSNE,
    CLUSTER: CLUSTER,
    ANNOTATION: ANNOTATION,
    DIMRED: DIMRED,
    SPECI: SPECI,
    TEMP: TEMP,
}

logger = logging.getLogger("checkatlas")


def get_workingdir(path: str) -> str:
    """Return the working_dir = path of search
    + working_dir
    with working_dir = checkatlas_files/

    Args:
        path (str): Search path for atlas given by user

    Returns:
        str: os.path.join(path, working_dir)
    """
    return os.path.join(path, WORKING_DIR)


def get_folder(path: str, key_folder: str) -> str:
    """Get the folder path giving the search path and
    the folder key in DICT_FOLDER

    Args:
        path (str): Search path for atlas given by user
        key_folder (str): key folder in the DICT_FOLDER
            example: ANNDATA, SUMMARY, UMAP

    Returns:
        str: the folder path
    """
    return os.path.join(get_workingdir(path), DICT_FOLDER[key_folder])


def checkatlas_folders(path: str) -> None:
    """Check in path if the different checkatlas folders exists.<br>
    Create them if needed.
    All folders are given by DICT_FOLDER

    Args:
        path (str): Search path for atlas given by user

    Returns:
        None: None
    """
    global_path = get_workingdir(path)
    if not os.path.exists(global_path):
        os.mkdir(global_path)

    for key_folder in DICT_FOLDER.keys():
        temp_path = os.path.join(global_path, key_folder)
        if not os.path.exists(temp_path):
            logger.debug(f"Create folder: {temp_path}")
            os.mkdir(temp_path)


def get_multiqc_folder(path: str) -> str:
    return os.path.join(path, MULTIQC_FOLDER)



import argparse
import logging
import os
import re
import warnings

import numpy as np
import pandas as pd
import scanpy as sc
from anndata import AnnData
from anndata import _io as _io
from sklearn.utils.fixes import _object_dtype_isnan

from . import cellranger, checkatlas
from .metrics import metrics
from .utils import files, folders

"""
Atlas module for AnnData (Scanpy)
All the function to screen the atlases
"""

ANNDATA_TYPE = "AnnData"
ANNDATA_EXTENSION = ".h5ad"

OBS_CLUSTERS = [
    "cell_type",
    "CellType",
    "celltype",
    "ann_finest_level",
    "cellranger_graphclust",
    "cellranger_kmeans",
    "seurat_clusters",
    "RNA_snn_res.0.5",
    "louvain",
    "leiden",
    "orig.ident",
]

OBSM_DIMRED = [
    "X_umap",
    "X_pca",
    "X-tsne",
]

OBS_QC = [
    "n_genes_by_counts",
    "total_counts",
    "pct_counts_mt",
    "pct_counts_ribo",
    "percent.mito",
    "percent.ribo",
    "dropouts",
    "nCount_SCT",
    "nFeature_SCT",
    "nCount_RNA",
    "nFeature_RNA",
    "nCount_HTO",
    "nFeature_HTO",
    "n_genes",
    "n_genes_by_counts",
    "total_counts",
    "total_counts_mt",
    "pct_counts_mt",
    "total_counts_ribo",
    "pct_counts_ribo",
]

CELLINDEX_HEADER = "cell_index"

logger = logging.getLogger("checkatlas")

warnings.simplefilter(action="ignore", category=FutureWarning)
warnings.simplefilter(action="ignore", category=UserWarning)
sc.settings.verbosity = 0


def detect_scanpy(atlas_path: str) -> dict:
    if atlas_path.endswith(ANNDATA_EXTENSION):
        atlas_info = dict()
        atlas_info[checkatlas.ATLAS_NAME_KEY] = os.path.splitext(
            os.path.basename(atlas_path)
        )[0]
        atlas_info[checkatlas.ATLAS_TYPE_KEY] = ANNDATA_TYPE
        atlas_info[checkatlas.ATLAS_EXTENSION_KEY] = ANNDATA_EXTENSION
        atlas_info[checkatlas.ATLAS_PATH_KEY] = atlas_path
        return atlas_info
    else:
        return dict()


def read_atlas(atlas_info: dict) -> AnnData:
    """
    Read Scanpy or Cellranger data : .h5ad or .h5

    Args:
        atlas_path (dict): info about the atlas

    Returns:
        AnnData: scanpy object from .h5ad
    """
    logger.info(
        f"Load {atlas_info[checkatlas.ATLAS_NAME_KEY]} "
        f"in {atlas_info[checkatlas.ATLAS_PATH_KEY]}"
    )
    try:
        if (
            atlas_info[checkatlas.ATLAS_TYPE_KEY]
            == cellranger.CELLRANGER_TYPE_CURRENT
        ):
            logger.debug(
                "Read Cellranger >= v3 results "
                f"{atlas_info[checkatlas.ATLAS_PATH_KEY]}"
            )
            adata = cellranger.read_cellranger_current(atlas_info)
        elif (
            atlas_info[checkatlas.ATLAS_TYPE_KEY]
            == cellranger.CELLRANGER_TYPE_OBSOLETE
        ):
            logger.debug(
                "Read Cellranger < v3 results "
                f"{atlas_info[checkatlas.ATLAS_PATH_KEY]}"
            )
            adata = cellranger.read_cellranger_obsolete(atlas_info)
        else:
            logger.debug(
                f"Read Scanpy file {atlas_info[checkatlas.ATLAS_PATH_KEY]}"
            )
            adata = sc.read_h5ad(atlas_info[checkatlas.ATLAS_PATH_KEY])
        return adata
    except _io.utils.AnnDataReadError:
        logger.warning(
            "AnnDataReadError, cannot read: "
            f"{atlas_info[checkatlas.ATLAS_PATH_KEY]}"
        )
        return dict()


def clean_scanpy_atlas(adata: AnnData, atlas_info: dict) -> AnnData:
    """
    Clean the Scanpy object to be sure to get all information out of it

    - Make var names unique
    - Make var unique for Raw matrix
    - If OBS_CLUSTERS are present and in int32 -> be sure to
    transform them in categorical

    Args:
        adata (AnnData): atlas to analyse
        atlas_info (dict): info on the atlas

    Returns:
        AnnData: cleaned atlas
    """
    logger.debug(f"Clean scanpy: {atlas_info[checkatlas.ATLAS_NAME_KEY]}")
    # Make var names unique
    list_var = adata.var_names
    if len(set(list_var)) == len(list_var):
        logger.debug("Var names unique")
    else:
        logger.debug(
            "Var names not unique, ran : adata.var_names_make_unique()"
        )
        adata.var_names_make_unique()
        # Test a second time if it is unique (sometimes it helps)
        list_var = adata.var_names
        if len(set(list_var)) == len(list_var):
            logger.debug("Var names unique")
        else:
            logger.debug(
                "Var names not unique, ran : adata.var_names_make_unique()"
            )
            adata.var_names_make_unique()
            # If it is still not unique, create unique var_names "by hand"
            list_var = adata.var_names
            if len(set(list_var)) == len(list_var):
                logger.debug("Var names unique")
            else:
                logger.debug(
                    "Var names not unique, ran : adata.var_names_make_unique()"
                )
                adata.var.index = [
                    x + "_" + str(i)
                    for i, x in zip(range(len(adata.var)), adata.var_names)
                ]
                list_var = adata.var_names
                if len(set(list_var)) == len(list_var):
                    logger.debug("Var names unique")
    # Make var unique for Raw matrix
    if adata.raw is not None:
        list_var = adata.raw.var_names
        if len(set(list_var)) == len(list_var):
            logger.debug("Var names for Raw unique, transform ")
        else:
            logger.debug("Var names for Raw not unique")
            adata.raw.var.index = [
                x + "_" + str(i)
                for i, x in zip(range(len(adata.raw.var)), adata.raw.var_names)
            ]
            list_var = adata.raw.var_names
            if len(set(list_var)) == len(list_var):
                logger.debug("Var names for Raw unique")

    # If OBS_CLUSTERS are present and in int32 -> be sure to
    # transform them in categorical
    for obs_key in adata.obs_keys():
        for obs_key_celltype in OBS_CLUSTERS:
            if obs_key_celltype in obs_key:
                if (
                    adata.obs[obs_key].dtype == np.int32
                    or adata.obs[obs_key].dtype == np.int64
                ):
                    adata.obs[obs_key] = pd.Categorical(adata.obs[obs_key])
    return adata


def get_viable_obs_qc(adata: AnnData, args: argparse.Namespace) -> list:
    """
    Search in obs_keys a match to OBS_QC values
    Extract sorted obs_keys in same order then OBS_QC

    Args:
        adata (AnnData): atlas to analyse
        args (argparse.Namespace): list of arguments from checkatlas workflow

    Returns:
        list: obs_keys
    """
    obs_keys = list()
    for obs_key in adata.obs_keys():
        if obs_key in args.qc_display:
            obs_keys.append(obs_key)
    return obs_keys


def get_viable_obs_annot(adata: AnnData, args: argparse.Namespace) -> list:
    """
    Search in obs_keys a match to OBS_CLUSTERS values
    ! Remove obs_key with only one category !
    Extract sorted obs_keys in same order then OBS_CLUSTERS

    Args:
        adata (AnnData): atlas to analyse
        args (argparse.Namespace): list of arguments from checkatlas workflow

    Returns:
        list: obs_keys
    """
    obs_keys = list()
    # Get keys from OBS_CLUSTERS
    for obs_key in adata.obs_keys():
        for obs_key_celltype in args.obs_cluster:
            if obs_key_celltype in obs_key:
                if type(adata.obs[obs_key].dtype) == pd.CategoricalDtype:
                    obs_keys.append(obs_key)
    # Remove keys with only one category and no NaN in the array
    obs_keys_final = list()
    for obs_key in obs_keys:
        annotations = adata.obs[obs_key]
        if not _object_dtype_isnan(annotations).any():
            categories_temp = annotations.cat.categories
            # remove nan if found
            categories = categories_temp.dropna()
            if True in categories.isin(["nan"]):
                index = categories.get_loc("nan")
                categories = categories.delete(index)
            # Add obs_key with more than one category (with Nan removed)
            if len(categories) != 1:
                logger.debug(
                    f"Add obs_key {obs_key} with cat {categories_temp}"
                )
                obs_keys_final.append(obs_key)
    return sorted(obs_keys_final)


def get_viable_obsm(adata: AnnData, args: argparse.Namespace) -> list:
    """
    TO DO
    Search viable obsm for dimensionality reduction metric
    calc.
    ! No filter on osbm is appled for now !
    Args:
        adata (AnnData): atlas to analyse
        args (argparse.Namespace): list of arguments from checkatlas workflow

    Returns:
        list: obsm_keys
    """
    obsm_keys = list()
    # for obsm_key in adata.obsm_keys():
    #   if obsm_key in args.obsm_dimred:
    obsm_keys = adata.obsm_keys()
    logger.debug(f"Add obsm {obsm_keys}")
    return obsm_keys


def create_summary_table(
    adata: AnnData, atlas_info: dict, args: argparse.Namespace
) -> None:
    """
    Create a table with all summarizing variables

    Args:
        adata (AnnData): atlas to analyse
        atlas_path (str): path of the atlas
        args (argparse.Namespace): list of arguments from checkatlas workflow
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    atlas_type = atlas_info[checkatlas.ATLAS_TYPE_KEY]
    atlas_path = atlas_info[checkatlas.ATLAS_PATH_KEY]
    logger.debug(f"Create Summary table for {atlas_name}")
    csv_path = files.get_file_path(
        atlas_name, folders.SUMMARY, checkatlas.TSV_EXTENSION, args.path
    )
    # Create summary table
    header = [
        "AtlasFileType",
        "NbCells",
        "NbGenes",
        "AnnData.raw",
        "AnnData.X",
        "File_extension",
        "File_path",
    ]
    df_summary = pd.DataFrame(index=[atlas_name], columns=header)
    df_summary["AtlasFileType"][atlas_name] = atlas_type
    df_summary["NbCells"][atlas_name] = adata.n_obs
    df_summary["NbGenes"][atlas_name] = adata.n_vars
    df_summary["AnnData.raw"][atlas_name] = adata.raw is not None
    df_summary["AnnData.X"][atlas_name] = adata.X is not None
    df_summary["File_extension"][atlas_name] = atlas_name
    df_summary["File_path"][atlas_name] = atlas_path
    df_summary.to_csv(csv_path, index=False, sep="\t")


def create_anndata_table(
    adata: AnnData, atlas_info: dict, args: argparse.Namespace
) -> None:
    """
    Create an html table with all AnnData arguments
    The html code will make all elements of the table visible in MultiQC
    Args:
        adata (AnnData): atlas to analyse
        atlas_info (dict): info on the atlas
        args (argparse.Namespace): list of arguments from checkatlas workflow
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]

    logger.debug(f"Create Adata table for {atlas_name}")
    csv_path = files.get_file_path(
        atlas_name, folders.ANNDATA, checkatlas.TSV_EXTENSION, args.path
    )
    # Create AnnData table
    header = ["atlas_obs", "obsm", "var", "varm", "uns"]
    df_summary = pd.DataFrame(index=[atlas_name], columns=header)
    # html_element = "<span class=\"label label-primary\">"
    # new_line = ''
    # for value in list(adata.obs.columns):
    #     new_line += html_element + value + "</span><br>"
    #     print(new_line)
    df_summary["atlas_obs"][atlas_name] = (
        "<code>"
        + "</code><br><code>".join(list(adata.obs.columns))
        + "</code>"
    )
    df_summary["obsm"][atlas_name] = (
        "<code>"
        + "</code><br><code>".join(list(adata.obsm_keys()))
        + "</code>"
    )
    df_summary["var"][atlas_name] = (
        "<code>" + "</code><br><code>".join(list(adata.var_keys())) + "</code>"
    )
    df_summary["varm"][atlas_name] = (
        "<code>"
        + "</code><br><code>".join(list(adata.varm_keys()))
        + "</code>"
    )
    df_summary["uns"][atlas_name] = (
        "<code>" + "</code><br><code>".join(list(adata.uns_keys())) + "</code>"
    )
    df_summary.to_csv(csv_path, index=False, quoting=False, sep="\t")


def create_qc_tables(
    adata: AnnData, atlas_info: dict, args: argparse.Namespace
) -> None:
    """
    Display the atlas QC table
    Search for the OBS variable which correspond to the toal_RNA, total_UMI,
     MT_ratio, RT_ratio

    Args:
        adata (AnnData): atlas to analyse
        atlas_info (dict): info on the atlas
        args (argparse.Namespace): list of arguments from checkatlas workflow
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    qc_path = files.get_file_path(
        atlas_name, folders.QC, checkatlas.TSV_EXTENSION, args.path
    )
    logger.debug(f"Create QC tables for {atlas_name}")
    qc_genes = []
    # mitochondrial genes
    adata.var["mt"] = adata.var_names.str.startswith("MT-")
    if len(adata.var[adata.var["mt"]]) != 0:
        qc_genes.append("mt")
        logger.debug(f"Mitochondrial genes in {atlas_name} for QC")
    else:
        logger.debug(f"No mitochondrial genes in {atlas_name} for QC")
    # ribosomal genes
    adata.var["ribo"] = adata.var_names.str.startswith(("RPS", "RPL"))
    if len(adata.var[adata.var["mt"]]) != 0:
        qc_genes.append("ribo")
        logger.debug(f"Ribosomal genes in {atlas_name} for QC")
    else:
        logger.debug(f"No ribosomal genes in {atlas_name} for QC")

    sc.pp.calculate_qc_metrics(
        adata,
        qc_vars=qc_genes,
        percent_top=None,
        log1p=False,
        inplace=True,
    )
    df_annot = adata.obs[get_viable_obs_qc(adata, args)]
    # Rank cell by qc metric
    for header in df_annot.columns:
        if header != CELLINDEX_HEADER:
            new_header = f"cellrank_{header}"
            df_annot = df_annot.sort_values(header, ascending=False)
            df_annot.loc[:, [new_header]] = range(1, adata.n_obs + 1)

    # Sample QC table when more cells than args.plot_celllimit are present
    df_annot = atlas_sampling(df_annot, "QC", args)
    df_annot.loc[:, [CELLINDEX_HEADER]] = range(1, len(df_annot) + 1)
    df_annot.to_csv(qc_path, index=False, quoting=False, sep="\t")


def create_qc_plots(
    adata: AnnData, atlas_info: dict, args: argparse.Namespace
) -> None:
    """
    Display the atlas QC plot
    Search for the OBS variable which correspond to the toal_RNA, total_UMI,
     MT_ratio, RT_ratio

    Args:
        adata (AnnData): atlas to analyse
        atlas_info (dict): info on the atlas
        args (argparse.Namespace): list of arguments from checkatlas workflow
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    sc.settings.figdir = folders.get_workingdir(args.path)
    sc.set_figure_params(dpi_save=80)
    qc_path = os.sep + atlas_name + checkatlas.QC_FIG_EXTENSION
    logger.debug(f"Create QC violin plot for {atlas_name}")
    # mitochondrial genes
    adata.var["mt"] = adata.var_names.str.startswith("MT-")
    # ribosomal genes
    adata.var["ribo"] = adata.var_names.str.startswith(("RPS", "RPL"))
    sc.pp.calculate_qc_metrics(
        adata,
        qc_vars=["mt", "ribo"],
        percent_top=None,
        log1p=False,
        inplace=True,
    )
    sc.pl.violin(
        adata,
        [
            "n_genes_by_counts",
            "total_counts",
            "pct_counts_mt",
            "pct_counts_ribo",
        ],
        jitter=0.4,
        multi_panel=True,
        show=False,
        save=qc_path,
    )


def create_umap_fig(
    adata: AnnData, atlas_info: dict, args: argparse.Namespace
) -> None:
    """
    Display the UMAP of celltypes
    Search for the OBS variable which correspond to the celltype annotation

    Args:
        adata (AnnData): atlas to analyse
        atlas_info (dict): info on the atlas
        args (argparse.Namespace): list of arguments from checkatlas workflow
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    sc.set_figure_params(dpi_save=150)
    # Search if umap reduction exists
    obsm_keys = get_viable_obsm(adata, args)
    r = re.compile(".*umap*.")
    obsm_umap_keys = list(filter(r.match, obsm_keys))
    if len(obsm_umap_keys) > 0:
        obsm_umap = obsm_umap_keys[0]
        logger.debug(
            f"Create UMAP figure for {atlas_name} with obsm={obsm_umap}"
        )
        # Set the umap to display
        if isinstance(adata.obsm[obsm_umap], pd.DataFrame):
            # Transform to numpy if it is a pandas dataframe
            adata.obsm["X_umap"] = adata.obsm[obsm_umap].to_numpy()
        else:
            adata.obsm["X_umap"] = adata.obsm[obsm_umap]
        # Setting up figures directory
        sc.settings.figdir = folders.get_workingdir(args.path)
        umap_path = os.sep + atlas_name + checkatlas.UMAP_EXTENSION
        # Exporting umap
        obs_keys = get_viable_obs_annot(adata, args)
        if len(obs_keys) != 0:
            sc.pl.umap(adata, color=obs_keys[0], show=False, save=umap_path)
        else:
            sc.pl.umap(adata, show=False, save=umap_path)


def create_tsne_fig(
    adata: AnnData, atlas_info: dict, args: argparse.Namespace
) -> None:
    """
    Display the TSNE of celltypes
    Search for the OBS variable which correspond to the celltype annotation

    Args:
        adata (AnnData): atlas to analyse
        atlas_info (dict): info on the atlas
        args (argparse.Namespace): list of arguments from checkatlas workflow
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    sc.set_figure_params(dpi_save=150)
    # Search if tsne reduction exists
    obsm_keys = get_viable_obsm(adata, args)
    r = re.compile(".*tsne*.")
    obsm_tsne_keys = list(filter(r.match, obsm_keys))
    if len(obsm_tsne_keys) > 0:
        obsm_tsne = obsm_tsne_keys[0]
        logger.debug(
            f"Create t-SNE figure for {atlas_name} with obsm={obsm_tsne}"
        )
        # Set the t-sne to display
        if isinstance(adata.obsm[obsm_tsne], pd.DataFrame):
            # Transform to numpy if it is a pandas dataframe
            adata.obsm["X_tsne"] = adata.obsm[obsm_tsne].to_numpy()
        else:
            adata.obsm["X_tsne"] = adata.obsm[obsm_tsne]
        # Setting up figures directory
        sc.settings.figdir = sc.settings.figdir = folders.get_workingdir(
            args.path
        )
        tsne_path = os.sep + atlas_name + checkatlas.TSNE_EXTENSION
        # Exporting tsne
        obs_keys = get_viable_obs_annot(adata, args)
        if len(obs_keys) != 0:
            sc.pl.tsne(adata, color=obs_keys[0], show=False, save=tsne_path)
        else:
            sc.pl.tsne(adata, show=False, save=tsne_path)


def create_metric_cluster(
    adata: AnnData, atlas_info: dict, args: argparse.Namespace
) -> None:
    """
    Calc clustering metrics

    Args:
        adata (AnnData): atlas to analyse
        atlas_info (dict): path of the atlas
        args (argparse.Namespace): list of arguments from checkatlas workflow
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    csv_path = files.get_file_path(
        atlas_name,
        folders.CLUSTER,
        checkatlas.TSV_EXTENSION,
        args.path,
    )
    header = ["Clust_Sample", "obs"] + args.metric_cluster
    df_cluster = pd.DataFrame(columns=header)
    obs_keys = get_viable_obs_annot(adata, args)
    obsm_keys = get_viable_obsm(adata, args)
    r = re.compile(".*umap*.")
    obsm_umap_keys = list(filter(r.match, obsm_keys))
    r = re.compile(".*tsne*.")
    obsm_tsne_keys = list(filter(r.match, obsm_keys))
    obsm_key_representation = ""
    if len(obsm_umap_keys) > 0:
        obsm_key_representation = obsm_umap_keys[0]
        print("reach", obsm_key_representation)
    elif len(obsm_tsne_keys) > 0:
        obsm_key_representation = obsm_tsne_keys[0]
        print("reach", obsm_key_representation)

    if len(obs_keys) > 0:
        logger.debug(f"Calc clustering metrics for {atlas_name}")
        for obs_key in obs_keys:
            dict_line = {
                "Clust_Sample": [atlas_name + "_" + obs_key],
                "obs": [obs_key],
            }
            for metric in args.metric_cluster:
                logger.debug(
                    f"Calc {metric} for {atlas_name} "
                    f"with obs {obs_key} and obsm {obsm_key_representation}"
                )
                metric_value = metrics.calc_metric_cluster_scanpy(
                    metric, adata, obs_key, obsm_key_representation
                )
                dict_line[metric] = metric_value
            df_line = pd.DataFrame(dict_line)
            df_cluster = pd.concat(
                [df_cluster, df_line], ignore_index=True, axis=0
            )
        df_cluster.to_csv(csv_path, index=False, sep="\t")
    else:
        logger.debug(f"No viable obs_key was found for {atlas_name}")


def create_metric_annot(
    adata: AnnData, atlas_info: dict, args: argparse.Namespace
) -> None:
    """
    Calc annotation metrics

    Args:
        adata (AnnData): atlas to analyse
        atlas_path (dict): path of the atlas
        args (argparse.Namespace): list of arguments from checkatlas workflow
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    csv_path = files.get_file_path(
        atlas_name,
        folders.ANNOTATION,
        checkatlas.TSV_EXTENSION,
        args.path,
    )
    header = ["Annot_Sample", "Reference", "obs"] + args.metric_annot
    df_annot = pd.DataFrame(columns=header)
    obs_keys = get_viable_obs_annot(adata, args)
    if len(obs_keys) > 1:
        logger.debug(f"Calc annotation metrics for {atlas_name}")
        ref_obs = obs_keys[0]
        for i in range(1, len(obs_keys)):
            obs_key = obs_keys[i]
            dict_line = {
                "Annot_Sample": [atlas_name + "_" + obs_key],
                "Reference": [ref_obs],
                "obs": [obs_key],
            }
            for metric in args.metric_annot:
                logger.debug(
                    f"Calc {metric} for {atlas_name} "
                    f"with obs {obs_key} vs ref_obs {ref_obs}"
                )
                metric_value = metrics.calc_metric_annot_scanpy(
                    metric, adata, obs_key, ref_obs
                )
                dict_line[metric] = metric_value
            df_line = pd.DataFrame(dict_line)
            df_annot = pd.concat(
                [df_annot, df_line], ignore_index=True, axis=0
            )
        df_annot.to_csv(csv_path, index=False, sep="\t")
    else:
        logger.debug(f"No viable obs_key was found for {atlas_name}")


def create_metric_dimred(
    adata: AnnData, atlas_info: dict, args: argparse.Namespace
) -> None:
    """
    Calc dimensionality reduction metrics

    Args:
        adata (AnnData): atlas to analyse
        atlas_info (dict): path of the atlas
        args (argparse.Namespace): list of arguments from checkatlas workflow
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    csv_path = files.get_file_path(
        atlas_name,
        folders.DIMRED,
        checkatlas.TSV_EXTENSION,
        args.path,
    )
    header = ["Dimred_Sample", "obsm"] + args.metric_dimred
    df_dimred = pd.DataFrame(columns=header)
    obsm_keys = get_viable_obsm(adata, args)
    if len(obsm_keys) > 0:
        logger.debug(f"Calc dim red metrics for {atlas_name}")
        for obsm_key in obsm_keys:
            dict_line = {
                "Dimred_Sample": [atlas_name + "_" + obsm_key],
                "obsm": [obsm_key],
            }
            for metric in args.metric_dimred:
                logger.debug(
                    f"Calc {metric} for {atlas_name} with obsm {obsm_key}"
                )
                metric_value = metrics.calc_metric_dimred(
                    metric, adata, obsm_key
                )
                dict_line[metric] = metric_value
            df_line = pd.DataFrame(dict_line)
            df_dimred = pd.concat(
                [df_dimred, df_line], ignore_index=True, axis=0
            )
        df_dimred.to_csv(csv_path, index=False, sep="\t")
    else:
        logger.debug(f"No viable obsm_key was found for {atlas_name}")


def atlas_sampling(
    df_annot: pd.DataFrame, type_df: str, args: argparse.Namespace
) -> pd.DataFrame:
    """
    If args.plot_celllimit != 0 and args.plot_celllimit < len(df_annot)
    The atlas qC table will be sampled for MultiQC

    Args:
        df_annot (pd.DataFrame): Table to sample
        type_df (str): type of table
        args (argparse.Namespace): arguments of checkatlas workflow

    Returns:
        pd.DataFrame: Sampled QC table
    """
    if args.plot_celllimit != 0 and args.plot_celllimit < len(df_annot):
        logger.debug(f"Sample {type_df} table with {len(df_annot)} cells")
        df_annot = df_annot.sample(args.plot_celllimit)
        logger.debug(f"{type_df} table sampled to {len(df_annot)} cells")
    return df_annot



import argparse
from importlib.resources import files

from checkatlas import atlas, checkatlas
from checkatlas.metrics import annot, cluster, dimred


def create_parser():
    parser = argparse.ArgumentParser(
        prog="checkatlas",
        usage="checkatlas [OPTIONS] process atlas_name your_search_folder/",
        description="CheckAtlas is a one liner tool to check the "
        "quality of your single-cell atlases. For "
        "every atlas, it produces the quality control "
        "tables and figures which can be then processed "
        "by multiqc. CheckAtlas is able to load Scanpy, "
        "Seurat, and CellRanger files.",
        epilog="Enjoy the checkatlas functionality!",
    )

    # All Program arguments
    # All Program arguments
    # main_options = parser.add_argument_group("Main arguments")
    parser.add_argument(
        "process",
        type=str,
        help="Required argument: Type of process to run"
        f" among {checkatlas.PROCESS_TYPE}",
        default="",
    )

    parser.add_argument(
        "atlas_name",
        type=str,
        help="Required argument: The name of the atlas to process."
        "Atlas_name should be found in one of the samplesheet provided to"
        "nf-checkatlas, or directly created by checkatlas.list_all_atlases()"
        " function",
        default=".",
    )

    parser.add_argument(
        "path",
        type=str,
        help="Required argument: Your folder containing "
        "Scanpy, CellRanger and Seurat atlasesv",
        default=".",
    )

    parser.add_argument(
        "-d",
        "--debug",
        action="store_true",
        help="Print out all debug messages.",
    )
    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=f"Checkatlas, version {get_version()}",
        help="Display checkatlas version.",
    )

    # Arguments linked to QC
    qc_options = parser.add_argument_group("QC options")
    qc_options.add_argument(
        "--qc_display",
        nargs="+",
        type=str,
        default=[
            "violin_plot",
            "total_counts",
            "n_genes_by_counts",
            "pct_counts_mt",
        ],
        help="List of QC to display. "
        "Available qc = violin_plot, total_counts, "
        "n_genes_by_counts, pct_counts_mt. "
        "Default: --qc_display violin_plot total_counts "
        "n_genes_by_counts pct_counts_mt",
    )
    qc_options.add_argument(
        "--plot_celllimit",
        type=int,
        default=0,
        help="Set the maximum number of cells"
        "to plot in QC, UMAP, t-SNE, etc...."
        "If plot_celllimit=0, no limit will"
        "be applied.",
    )

    # Arguments linked to metric
    metric_options = parser.add_argument_group("Metric options")
    metric_options.add_argument(
        "--obs_cluster",
        nargs="+",
        type=str,
        default=atlas.OBS_CLUSTERS,
        help="List of obs from the adata file to "
        "use in the clustering metric calculus."
        "Example: --obs_cluster celltype leuven seurat_clusters",
    )
    metric_options.add_argument(
        "--metric_cluster",
        nargs="+",
        type=str,
        default=["davies_bouldin"],
        # default=["silhouette", "davies_bouldin"],
        # default=[""],
        help="Specify the list of clustering metrics to calculate.\n"
        "   Example: --metric_cluster silhouette davies_bouldin\n"
        f"   List of cluster metrics: {cluster.__all__}",
    )
    metric_options.add_argument(
        "--metric_annot",
        nargs="+",
        type=str,
        # default=[],
        default=["rand_index"],
        help=f"Specify the list of clustering metrics to calculate."
        f"   Example: --metric_annot rand_index"
        f"   List of annotation metrics: {annot.__all__}",
    )
    metric_options.add_argument(
        "--metric_dimred",
        nargs="+",
        type=str,
        default=[],
        # default=["kruskal_stress"],
        help="Specify the list of dimensionality reduction "
        "metrics to calculate.\n"
        "   Example: --metric_dimred kruskal_stress\n"
        f"   List of dim. red. metrics: {dimred.__all__}",
    )
    return parser


def get_version():
    """
    Get version of checkatlas from checkatlas/VERSION file
    :return: checkatlas version
    """
    version_file = files(__package__).joinpath("VERSION")
    with open(version_file) as file:
        version = file.readline()
        return version



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['checkatlas',
 'checkatlas.metrics',
 'checkatlas.metrics.annot',
 'checkatlas.metrics.cluster',
 'checkatlas.metrics.dimred',
 'checkatlas.metrics.specificity',
 'checkatlas.utils']

package_data = \
{'': ['*']}

install_requires = \
['llvmlite>=0.42.0,<0.43.0',
 'numba>=0.59.0,<0.60.0',
 'numpy>=1.23.5,<2.0.0',
 'poetry>=1.5.1,<2.0.0',
 'rpy2>=3.5.10,<4.0.0',
 'scanpy>=1.9.1,<2.0.0',
 'scikit-learn>=1.4.1,<2.0.0',
 'types-pyyaml>=6.0.12.6,<7.0.0.0']

entry_points = \
{'console_scripts': ['checkatlas = checkatlas.__main__:main',
                     'checkatlas-workflow = '
                     'checkatlas.checkatlas_workflow:main']}

setup_kwargs = {
    'name': 'checkatlas',
    'version': '0.4.10',
    'description': 'One liner tool to check the quality of your single-cell atlases.',
    'long_description': "# ![CheckAtlas](docs/images/checkatlas_logo.png) \n\n\n![PyPI](https://img.shields.io/pypi/v/checkatlas)\n![PyPI - Downloads](https://img.shields.io/pypi/dw/checkatlas)\n![PyPI - License](https://img.shields.io/pypi/l/checkatlas)\n[![install with bioconda](https://img.shields.io/badge/install%20with-bioconda-brightgreen.svg?style=flat)](http://bioconda.github.io/recipes/checkatlas/README.html)\n\n[![codecov](https://codecov.io/gh/becavin-lab/checkatlas/branch/main/graph/badge.svg?token=checkatlas_token_here)](https://codecov.io/gh/becavin-lab/checkatlas)\n[![CI](https://github.com/becavin-lab/checkatlas/actions/workflows/tests.yml/badge.svg)](https://github.com/becavin-lab/checkatlas/actions/workflows/tests.yml)\n[![Documentation Status](https://readthedocs.org/projects/checkatlas/badge/?version=latest)](https://checkatlas.readthedocs.io/en/latest/?badge=latest)\n[![Gitter](https://badges.gitter.im/checkatlas/checkatlas.svg)](https://app.gitter.im/#/room/!KpJcsVTOlGjwJgtLwF:gitter.im)\n\n![Static Badge](https://img.shields.io/badge/Packaging-Poetry-blue)\n![Static Badge](https://img.shields.io/badge/Docs-Mkdocs-red)\n![Static Badge](https://img.shields.io/badge/Linting-flake8%20black%20mypy-yellow)\n\nCheckAtlas is a one liner tool to check the quality of your single-cell atlases. For every atlas, it produces the\nquality control tables and figures which can be then processed by multiqc. CheckAtlas is able to check the quality of Scanpy, Seurat,\nand CellRanger files.\n\nMore information on the [read the doc page](https://checkatlas.readthedocs.io/en/latest/)\n\n\n## Summary\n\nPowered by nextflow, checkatlas can be ran in one command line:\n\n```bash\nnextflow run nf-core-checkatlas -r dev --path search_folder/\n```\n\nThe checkatlas workflow start with a fast crawl through your working directory. It detects Seurat (.rds), Scanpy (.h5ad) or cellranger (.h5) atlas files.\n\nThen, it goes through all atlas files and produce summary information:\n\n- All basic QC (nRNA, nFeature, ratio_mito)\n- General information (nbcells, nbgenes, nblayers)\n- All elements in atlas files (obs, obsm, uns, var, varm)\n- Reductions (pca, umap, tsne)\n- All metrics (clustering, annotation, dimreduction, specificity)\n\nAll tables and figs are saved in the **checkatlas_files** folder in your search folder.\n\nA single html report is produced, using MultiQC, in **checkatlas_files/Checkatlas-MultiQC.html**.\n\n![Checkatlas workflow](docs/images/checkatlas_workflow.png)\n\n\n## Examples\n\n- Evaluate and compare different scanpy atlases:\n[Example 1](https://checkatlas.readthedocs.io/en/latest/examples/CheckAtlas_example_1/Checkatlas_MultiQC.html)\n\n- Evaluate different version of one atlas:\n[Example 2](https://checkatlas.readthedocs.io/en/latest/examples/CheckAtlas_example_2/Checkatlas_MultiQC.html)\n\n- Evaluate Scanpy, Seurat and CellRanger objects in your folder:\n[Example 3](https://checkatlas.readthedocs.io/en/latest/examples/CheckAtlas_example_3/Checkatlas_MultiQC.html)\n\n- Evaluate an integrated Scanpy atlas with the corresponding raw CellRanger atlases:\n[Example 4](https://checkatlas.readthedocs.io/en/latest/examples/CheckAtlas_example_4/Checkatlas_MultiQC.html)\n\n- Evaluate different Cellranger atlases with multiple chemistry version and cellranger version:\n[Example 5](https://checkatlas.readthedocs.io/en/latest/examples/CheckAtlas_example_5/Checkatlas_MultiQC.html)\n\n\n## Installation\n\nCheckAtlas is in two parts. The checkatlas pythn module which can be downloaded with PyPi, and the checkatlas workflow which can be downloaded with nextflow.\n\n```bash\npip install checkatlas\n```\n\n```bash\nnextflow pull becavin-lab/nf-core-checkatlas\n```\n\nYou need also to install a version of MultiQC with checkatlas capability (for the moment). This version of MultiQC is available at checkatlas branch of github.com:becavin-lab/MultiQC.\n\n```bash\ngit clone git@github.com:becavin-lab/MultiQC.git\ncd MultiQC/\ngit checkout checkatlas\npip install .\n```\n\nFinally, checkatlas comes with rpy2 to perform the interface between python and R. But, it does not automatically install Seurat. So if you want to screen Seurat atlases you need to perfrom this last installation\n\n```bash\n% R\n> install.packages('Seurat')\n> library(Seurat)\n```\n\n\n## Development\n\nThis project is in a very early development phase. All helpers are welcome. Please contact us or submit an issue.\n\nRead the [CONTRIBUTING.md](docs/contributing.md) file.\n\nCheckatlas has two repositories:\n- [The checkatlas module](https://github.com/becavin-lab/checkatlas)\n- [The checkatlas nextflow workflow](https://github.com/becavin-lab/nf-core-checkatlas)\n\nIt has a module on MultiQC\n- [MultiQC checkatlas branch](https://github.com/becavin-lab/MultiQC)\n\nThe checkatlas package is available on PyPI\n- [Checkatlas PyPI](https://pypi.org/project/checkatlas/)\n\nThe bioconda recipe has been submitted\n- [Checkatlas bioconda recipe](https://github.com/drbecavin/bioconda-recipes)\n\n\n\nProject developed thanks to the project template : (https://github.com/rochacbruno/python-project-template/)\n\n",
    'author': 'becavin-lab',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://checkatlas.readthedocs.io/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import argparse
import logging
import os
import re
import warnings

import pandas as pd
import rpy2.robjects as ro
import rpy2.robjects as robjects
import rpy2.robjects.packages as rpackages
from rpy2.rinterface_lib.sexp import NULLType
from rpy2.robjects import pandas2ri
from rpy2.robjects.methods import RS4
from rpy2.robjects.packages import importr
from rpy2.robjects.vectors import FactorVector, StrVector

from checkatlas import atlas, checkatlas
from checkatlas.metrics import metrics
from checkatlas.utils import folders

"""
Module for management of Atlas n Seurat format

- Adata and summary tables are created from seuratà
- QC are created using Seurat
- UMAP, t-SNE are created using Seurat
- For creating metrics tables, obs tables are
  first converted from R to python and then
  scikitlearn is used.
"""

logger = logging.getLogger("checkatlas")
warnings.filterwarnings(action="ignore", message="R[write to console]")

SEURAT_TYPE = "Seurat"
SEURAT_EXTENSION = ".rds"


SCANPY_TO_SEURAT_OBS = {
    "total_counts": "nCount_RNA",
    "n_genes_by_counts": "nFeature_RNA",
    "pct_counts_mt": "percent.mito",
    "pct_counts_ribo": "percent.ribo",
    "violin_plot": "violin_plot",
}

SEURAT_TO_SCANPY_OBS = {
    "nCount_RNA": "total_counts",
    "nFeature_RNA": "n_genes_by_counts",
    "percent.mito": "pct_counts_mt",
    "percent.ribo": "pct_counts_ribo",
}


def detect_seurat(atlas_path: str) -> dict:
    if atlas_path.endswith(SEURAT_EXTENSION):
        atlas_info = dict()
        atlas_info[checkatlas.ATLAS_NAME_KEY] = os.path.splitext(
            os.path.basename(atlas_path)
        )[0]
        atlas_info[checkatlas.ATLAS_TYPE_KEY] = SEURAT_TYPE
        atlas_info[checkatlas.ATLAS_EXTENSION_KEY] = SEURAT_EXTENSION
        atlas_info[checkatlas.ATLAS_PATH_KEY] = atlas_path
        return atlas_info
    else:
        return dict()


def check_seurat_install() -> None:
    """Check if Seurat is installed, run installation if not"""
    # import R's utility package
    utils = rpackages.importr("utils")
    # select a mirror for R packages
    utils.chooseCRANmirror(ind=1)  # select the first mirror in the list
    # R package names
    packnames = ("Seurat", "SeuratObject")
    # Selectively install what needs to be install.
    # We are fancy, just because we can.
    names_to_install = [x for x in packnames if not rpackages.isinstalled(x)]
    if len(names_to_install) > 0:
        # create personal library
        rcode = """dir.create(Sys.getenv("R_LIBS_USER"), recursive = TRUE)"""
        robjects.r(rcode)
        # add to the path
        rcode = """.libPaths(Sys.getenv("R_LIBS_USER"))"""
        robjects.r(rcode)
        logger.debug(f"Set Rlibpaths: {robjects.r(rcode)}")
        utils.install_packages(StrVector(names_to_install))


def read_atlas(atlas_info: dict) -> RS4:
    """Read Seurat object in python using rpy2

    Args:
        atlas_path (str): _description_

    Returns:
        RS4: _description_
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    atlas_path = atlas_info[checkatlas.ATLAS_PATH_KEY]
    logger.info(f"Load {atlas_name} in " f"{atlas_path}")
    rcode = f'readRDS("{atlas_path}")'
    seurat = robjects.r(rcode)
    rclass = robjects.r["class"]
    if rclass(seurat)[0] == "Seurat":
        importr("Seurat")
        return seurat
    else:
        logger.info(f"{atlas_name} is not a Seurat object")
        return None


def get_viable_obs_qc(seurat: RS4, args: argparse.Namespace) -> list:
    """
    Search in obs_keys a match to OBS_QC values
    Extract sorted obs_keys in same order then OBS_QC

    Args:
        seurat (RS4): _description_
        args (argparse.Namespace): _description_

    Returns:
        list: _description_
    """
    r_obs = robjects.r(
        "obs <- function(seurat){ return(colnames(seurat@meta.data))}"
    )
    obs_keys = list()
    for obs_qc in args.qc_display:
        obs_qc = SCANPY_TO_SEURAT_OBS[obs_qc]
        if obs_qc in r_obs(seurat):
            obs_keys.append(obs_qc)
    return obs_keys


def get_viable_obs_annot(seurat: RS4, args: argparse.Namespace) -> list:
    """
    Search in obs_keys a match to OBS_CLUSTERS values
    ! Remove obs_key with only one category !
    Extract sorted obs_keys in same order then OBS_CLUSTERS

    Args:
        seurat (RS4): _description_
        args (argparse.Namespace): _description_

    Returns:
        list: _description_
    """
    obs_keys = list()
    r_obs = robjects.r(
        "obs <- function(seurat){ return(colnames(seurat@meta.data))}"
    )
    obs_key_seurat = r_obs(seurat)
    r_annot = robjects.r(
        "type <- function(seurat, obs_key){ "
        "return(seurat[[obs_key]][[obs_key]])}"
    )
    # Get keys from OBS_CLUSTERS
    for obs_key in obs_key_seurat:
        for obs_key_celltype in args.obs_cluster:
            if obs_key_celltype in obs_key:
                if isinstance(r_annot(seurat, obs_key), FactorVector):
                    obs_keys.append(obs_key)
    # Remove keys with only one category
    obs_keys_final = list()
    for obs_key in obs_keys:
        annotations = r_annot(seurat, obs_key)
        if len(annotations.levels) != 1:
            logger.debug(
                f"Add obs_key {obs_key} with cat {annotations.levels}"
            )
            obs_keys_final.append(obs_key)
    return sorted(obs_keys_final)


def get_viable_obsm(seurat, args):
    """
    Search viable obsm for dimensionality reduction metric
    calc.
    ! No filter on osbm is appled for now !
    :param seurat:
    :param args:
    :return:
    """
    obsm_keys = list()
    # for obsm_key in adata.obsm_keys():
    #   if obsm_key in args.obsm_dimred:
    r_obsm = robjects.r(
        "f<-function(seurat){return(names(seurat@reductions))}"
    )
    obsm_keys_r = r_obsm(seurat)
    obsm_keys = list()
    for obsm_key in obsm_keys_r:
        print(obsm_key)
        obsm_keys.append(obsm_key)
    logger.debug(f"Add obsm {obsm_keys}")
    return obsm_keys


def create_summary_table(
    seurat: RS4, atlas_info: dict, args=argparse.Namespace
) -> None:
    """
    Create a table with all interesting variables
    :param seurat:
    :param atlas_name:
    :param csv_path:
    :return:
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    logger.debug(f"Create Summary table for {atlas_name}")
    csv_path = os.path.join(
        folders.get_folder(args.path, folders.SUMMARY),
        atlas_name + checkatlas.TSV_EXTENSION,
    )
    # Create summary table
    header = [
        "AtlasFileType",
        "NbCells",
        "NbGenes",
        "AnnData.raw",
        "AnnData.X",
        "File_extension",
        "File_path",
    ]
    r_nrow = robjects.r["nrow"]
    r_ncol = robjects.r["ncol"]
    ncells = r_ncol(seurat)[0]
    ngenes = r_nrow(seurat)[0]
    x_raw = False
    x_norm = True
    df_summary = pd.DataFrame(index=[atlas_name], columns=header)
    df_summary["AtlasFileType"][atlas_name] = atlas_info[
        checkatlas.ATLAS_TYPE_KEY
    ]
    df_summary["NbCells"][atlas_name] = ncells
    df_summary["NbGenes"][atlas_name] = ngenes
    df_summary["AnnData.raw"][atlas_name] = x_raw
    df_summary["AnnData.X"][atlas_name] = x_norm
    df_summary["File_extension"][atlas_name] = atlas_info[
        checkatlas.ATLAS_EXTENSION_KEY
    ]
    df_summary["File_path"][atlas_name] = atlas_info[checkatlas.ATLAS_PATH_KEY]
    df_summary.to_csv(csv_path, index=False, sep="\t")


def create_anndata_table(
    seurat: RS4, atlas_info: dict, args=argparse.Namespace
) -> None:
    """
    Create a table with all AnnData-like arguments in Seurat object
    :param seurat:
    :param atlas_name:
    :param atlas_path:
    :return:
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    logger.debug(f"Create Adata table for {atlas_name}")
    csv_path = os.path.join(
        folders.get_folder(args.path, folders.ANNDATA),
        atlas_name + checkatlas.TSV_EXTENSION,
    )
    # Create AnnData table
    header = ["atlas_obs", "obsm", "var", "varm", "uns"]
    df_summary = pd.DataFrame(index=[atlas_name], columns=header)

    # Create r_functions
    r_obs = robjects.r(
        "obs <- function(seurat){ return(colnames(seurat@meta.data))}"
    )
    r_obsm = robjects.r(
        "f<-function(seurat){return(names(seurat@reductions))}"
    )
    r_uns = robjects.r(
        "uns <- function(seurat){ return(colnames(seurat@misc))}"
    )

    obs_list = r_obs(seurat)
    obsm_list = r_obsm(seurat)
    var_list = [""]
    varm_list = [""]
    uns_list = [""]
    if not isinstance(r_uns(seurat), NULLType):
        uns_list = r_uns(seurat)

    df_summary["atlas_obs"][atlas_name] = (
        "<code>" + "</code><br><code>".join(obs_list) + "</code>"
    )
    df_summary["obsm"][atlas_name] = (
        "<code>" + "</code><br><code>".join(obsm_list) + "</code>"
    )
    df_summary["var"][atlas_name] = (
        "<code>" + "</code><br><code>".join(var_list) + "</code>"
    )
    df_summary["varm"][atlas_name] = (
        "<code>" + "</code><br><code>".join(varm_list) + "</code>"
    )
    df_summary["uns"][atlas_name] = (
        "<code>" + "</code><br><code>".join(uns_list) + "</code>"
    )
    df_summary.to_csv(csv_path, index=False, quoting=False, sep="\t")


def create_qc_tables(
    seurat: RS4, atlas_info: dict, args=argparse.Namespace
) -> None:
    """
    Display the atlas QC of seurat
    Search for the metadata variable which correspond
    to the total_RNA, total_UMI, MT_ratio, RT_ratio
    :param path:
    :param adata:
    :param atlas_name:
    :param atlas_path:
    :return:
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    qc_path = os.path.join(
        folders.get_folder(args.path, folders.QC),
        atlas_name + checkatlas.TSV_EXTENSION,
    )
    logger.debug(f"Create QC tables for {atlas_name}")
    obs_keys = get_viable_obs_qc(seurat, args)
    r_meta = robjects.r("obs <- function(seurat){ return(seurat@meta.data)}")
    r_metadata = r_meta(seurat)
    with (ro.default_converter + pandas2ri.converter).context():
        df_metadata = ro.conversion.get_conversion().rpy2py(r_metadata)
        df_annot = df_metadata[obs_keys]
        # rename columns with scanpy names
        new_columns = list()
        for column in df_annot.columns:
            new_columns.append(SEURAT_TO_SCANPY_OBS[column])
        df_annot.columns = new_columns

        # Rank cell by qc metric
        for header in df_annot.columns:
            if header != atlas.CELLINDEX_HEADER:
                new_header = f"cellrank_{header}"
                df_annot = df_annot.sort_values(header, ascending=False)
                df_annot.loc[:, [new_header]] = range(1, len(df_annot) + 1)

        # Sample QC table when more cells than args.plot_celllimit are present
        df_annot = atlas.atlas_sampling(df_annot, "QC", args)
        df_annot.loc[:, [atlas.CELLINDEX_HEADER]] = range(1, len(df_annot) + 1)
        df_annot.to_csv(qc_path, index=False, quoting=False, sep="\t")


def create_qc_plots(
    seurat: RS4, atlas_info: dict, args=argparse.Namespace
) -> None:
    """
    Display the atlas QC
    Search for the OBS variable which correspond to the toal_RNA, total_UMI,
     MT_ratio, RT_ratio
    :param path:
    :param adata:
    :param atlas_name:
    :param atlas_path:
    :return:
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    qc_path = os.path.join(
        folders.get_folder(args.path, folders.QC_FIG),
        atlas_name + checkatlas.QC_FIG_EXTENSION,
    )
    logger.debug(f"Create QC violin plot for {atlas_name}")
    importr("ggplot2")
    r_cmd = (
        "vln_plot <- function(seurat, obs, qc_path){"
        "vln <- VlnPlot(seurat, features = obs, ncol = length(obs));"
        "ggsave(qc_path, vln, width = 10, "
        "height = 4, dpi = 150)}"
    )
    r_violin = robjects.r(r_cmd)
    obs_keys = list(SEURAT_TO_SCANPY_OBS.keys())
    r_obs = robjects.StrVector(obs_keys)
    r_violin(seurat, r_obs, qc_path)


def create_umap_fig(
    seurat: RS4, atlas_info: dict, args=argparse.Namespace
) -> None:
    """
    Display the UMAP of celltypes
    Search for the OBS variable which correspond to the celltype annotation
    :param path:
    :param adata:
    :param atlas_name:
    :param atlas_path:
    :return:
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    # Search if tsne reduction exists
    r = re.compile(".*umap*.")
    r_names = robjects.r["names"]
    obsm_list = r_names(seurat)
    importr("ggplot2")
    if len(list(filter(r.match, obsm_list))) > 0:
        logger.debug(f"Create UMAP figure for {atlas_name}")
        # Setting up figures directory
        umap_path = os.path.join(
            folders.get_folder(args.path, folders.UMAP),
            atlas_name + checkatlas.UMAP_EXTENSION,
        )
        # Exporting umap
        obs_keys = get_viable_obs_annot(seurat, args)
        r_cmd = (
            "umap <- function(seurat, obs_key, umap_path){"
            "umap_plot <- DimPlot(seurat, group.by = obs_key, "
            'reduction = "umap");'
            "ggsave(umap_path, umap_plot, width = 10, "
            "height = 6, dpi = 76)}"
        )
        r_umap = robjects.r(r_cmd)
        r_umap(seurat, obs_keys[0], umap_path)


def create_tsne_fig(
    seurat: RS4, atlas_info: dict, args=argparse.Namespace
) -> None:
    """
    Display the TSNE of celltypes
    Search for the OBS variable which correspond to the celltype annotation
    :param path:
    :param adata:
    :param atlas_name:
    :param atlas_path:
    :return:
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    # Search if tsne reduction exists
    r = re.compile(".*tsne*.")
    r_names = robjects.r["names"]
    obsm_list = r_names(seurat)
    importr("ggplot2")
    if len(list(filter(r.match, obsm_list))) > 0:
        logger.debug(f"Create t-SNE figure for {atlas_name}")
        # Setting up figures directory
        tsne_path = os.path.join(
            folders.get_folder(args.path, folders.TSNE),
            atlas_name + checkatlas.TSNE_EXTENSION,
        )
        # Exporting tsne
        obs_keys = get_viable_obs_annot(seurat, args)
        r_cmd = (
            "tsne <- function(seurat, obs_key, tsne_path){"
            "tsne_plot <- DimPlot(seurat, group.by = obs_key, "
            'reduction = "tsne");'
            "ggsave(tsne_path, tsne_plot, width = 10, "
            "height = 6, dpi = 76)}"
        )
        r_tsne = robjects.r(r_cmd)
        r_tsne(seurat, obs_keys[0], tsne_path)


def create_metric_cluster(
    seurat: RS4, atlas_info: dict, args=argparse.Namespace
) -> None:
    """
    Calc clustering metrics
    :param seurat:
    :param atlas_path:
    :param atlas_info:
    :param args:
    :return:
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    csv_path = os.path.join(
        folders.get_folder(args.path, folders.CLUSTER),
        atlas_name + checkatlas.TSV_EXTENSION,
    )
    header = ["Clust_Sample", "obs"] + args.metric_cluster
    df_cluster = pd.DataFrame(columns=header)
    obs_keys = get_viable_obs_annot(seurat, args)
    obsm_key_representation = "umap"
    if len(obs_keys) > 0:
        logger.debug(f"Calc clustering metrics for {atlas_name}")
        for obs_key in obs_keys:
            dict_line = {
                "Clust_Sample": [atlas_name + "_" + obs_key],
                "obs": [obs_key],
            }
            for metric in args.metric_cluster:
                logger.debug(
                    f"Calc {metric} for {atlas_name} "
                    f"with obs {obs_key} and obsm {obsm_key_representation}"
                )
                metric_value = metrics.calc_metric_cluster_seurat(
                    metric, seurat, obs_key, obsm_key_representation
                )
                dict_line[metric] = metric_value
            df_line = pd.DataFrame(dict_line)
            df_cluster = pd.concat(
                [df_cluster, df_line], ignore_index=True, axis=0
            )
        df_cluster.to_csv(csv_path, index=False, sep="\t")
    else:
        logger.debug(f"No viable obs_key was found for {atlas_name}")


def create_metric_annot(
    seurat: RS4, atlas_info: dict, args=argparse.Namespace
) -> None:
    """
    Calc annotation metrics
    :param adata:
    :param atlas_path:
    :param atlas_info:
    :param args:
    :return:
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    csv_path = os.path.join(
        folders.get_folder(args.path, folders.ANNOTATION),
        atlas_name + checkatlas.TSV_EXTENSION,
    )
    header = ["Annot_Sample", "Reference", "obs"] + args.metric_annot
    df_annot = pd.DataFrame(columns=header)
    obs_keys = get_viable_obs_annot(seurat, args)
    if len(obs_keys) > 1:
        logger.debug(f"Calc annotation metrics for {atlas_name}")
        if len(obs_keys) != 0:
            ref_obs = obs_keys[0]
            for i in range(1, len(obs_keys)):
                obs_key = obs_keys[i]
                dict_line = {
                    "Annot_Sample": [atlas_name + "_" + obs_key],
                    "Reference": [ref_obs],
                    "obs": [obs_key],
                }
                for metric in args.metric_annot:
                    logger.debug(
                        f"Calc {metric} for {atlas_name} "
                        f"with obs {obs_key} vs ref_obs {ref_obs}"
                    )
                    metric_value = metrics.calc_metric_annot_seurat(
                        metric, seurat, obs_key, ref_obs
                    )
                    dict_line[metric] = metric_value
                df_line = pd.DataFrame(dict_line)
                df_annot = pd.concat(
                    [df_annot, df_line], ignore_index=True, axis=0
                )
            df_annot.to_csv(csv_path, index=False, sep="\t")
    else:
        logger.debug(f"No viable obs_key was found for {atlas_name}")


def create_metric_dimred(
    seurat: RS4, atlas_info: dict, args=argparse.Namespace
) -> None:
    """
    Calc dimensionality reduction metrics
    :param adata:
    :param atlas_path:
    :param atlas_info:
    :param args:
    :return:
    """
    atlas_name = atlas_info[checkatlas.ATLAS_NAME_KEY]
    csv_path = os.path.join(
        folders.get_folder(args.path, folders.DIMRED),
        atlas_name + checkatlas.TSV_EXTENSION,
    )
    header = ["Dimred_Sample", "obsm"] + args.metric_dimred
    df_dimred = pd.DataFrame(columns=header)
    # r_reduction = robjects.r(
    #     "reduc <- function(seurat, obsm_key){"
    #     " return(Embeddings(object = seurat, reduction = obsm_key))}"
    # )
    obsm_keys = get_viable_obsm(seurat, args)
    if len(obsm_keys) > 0:
        logger.debug(f"Calc dim red metrics for {atlas_name}")
        for obsm_key in obsm_keys:
            dict_line = {
                "Dimred_Sample": [atlas_name + "_" + obsm_key],
                "obsm": [obsm_key],
            }
            for metric in args.metric_dimred:
                logger.debug(
                    f"Calc {metric} for {atlas_name} with obsm {obsm_key}"
                )
                # r_countmatrix = robjects.r(
                #     "mat <- function(seurat)
                #     { return(seurat@assays$RNA@counts)}"
                # )
                # high_dim_counts = ro.conversion.rpy2py(r_countmatrix(seurat))
                # low_dim_counts = ro.conversion.rpy2py(
                #    r_reduction(seurat, obsm_key)
                # )
                # metric_value = metrics.calc_metric_dimred(
                # metric, high_dim_counts, low_dim_counts)
                logger.warning(
                    "!!! Dim reduction metrics not available for Seurat"
                    " at the moment !!!"
                )
                # metric_value = -1
                # dict_line[metric] = str(metric_value)
            df_line = pd.DataFrame(dict_line)
            df_dimred = pd.concat(
                [df_dimred, df_line], ignore_index=True, axis=0
            )
        df_dimred.to_csv(csv_path, index=False, sep="\t")
    else:
        logger.debug(f"No viable obsm_key was found for {atlas_name}")
